@@ -251,7 +251,7 @@ class _BayesMixedGLM(base.Model):
         if not sparse.issparse(exog_vc):
             exog_vc = sparse.csr_matrix(exog_vc)
 
-        ident = ident.astype(np.int)
+        ident = ident.astype(int)
         vcp_p = float(vcp_p)
         fe_p = float(fe_p)
 
@@ -443,7 +443,7 @@ class _BayesMixedGLM(base.Model):
             mat = patsy.dmatrix(fml, data, return_type='dataframe')
             exog_vc.append(mat)
             vcp_names.append(na)
-            ident.append(j * np.ones(mat.shape[1], dtype=np.integer))
+            ident.append(j * np.ones(mat.shape[1], dtype=np.int_))
             j += 1
         exog_vc = pd.concat(exog_vc, axis=1)
         vc_names = exog_vc.columns.tolist()
